#import "KinemeTextureRenderer.h"

@implementation KinemeTextureRenderer

-(id)initWithPixelsWide:(unsigned int)width pixelsHigh:(unsigned int)height
{
	// check parameters.  width/height less than 16 aren't valid
	if( (width < 16) || (height < 16) )
	{
		[self release];
		return nil;
	}
	
	if(self = [super init])
	{
		_width=width;
		_height=height;
		
		// create scratch buffers.  should be page-aligned and rowbytes should be a multiple of 64
		_scratchBufferRowBytes = (width * 4 + 63) & ~63;
		_scratchBufferPtr = valloc(height * _scratchBufferRowBytes);
		if(_scratchBufferPtr == NULL)
		{
			NSLog(@"Failed to allocate scratch buffer");
			[self release];
			return nil;
		}
		_scratchBufferPtrFlip = valloc(height * _scratchBufferRowBytes);
		if(_scratchBufferPtrFlip == NULL)
		{
			NSLog(@"Failed to allocate second scratch buffer");
			[self release];
			return nil;
		}
		_colorspace = CGColorSpaceCreateDeviceRGB();
	}
	
	return self;
}

-(QCGLCGImage*) QCGLCGImageWithModule:(module::Module *)module type:(int)type
{
	utils::NoiseMap noiseMap;

	switch(type)
	{
		case 0:
		{
			utils::NoiseMapBuilderPlane builder;
			builder.SetBounds(-1.0, 1.0, -1.0, 1.0);
			builder.EnableSeamless(false);
			builder.SetDestSize(_width,_height);
			builder.SetSourceModule(*module);
			builder.SetDestNoiseMap(noiseMap);
			builder.Build();
			break;
		}
		case 1:
		{
			utils::NoiseMapBuilderPlane builder;
			builder.SetBounds(-1.0, 1.0, -1.0, 1.0);
			builder.EnableSeamless(true);
			builder.SetDestSize(_width,_height);
			builder.SetSourceModule(*module);
			builder.SetDestNoiseMap(noiseMap);
			builder.Build();
			break;
		}
		case 2:
		{
			utils::NoiseMapBuilderSphere builder;
			builder.SetBounds(-90.0, 90.0, -180.0, 180.0); // degrees
			builder.SetDestSize(_width,_height);
			builder.SetSourceModule(*module);
			builder.SetDestNoiseMap(noiseMap);
			builder.Build();
			break;
		}
	}

	utils::RendererImage textureRenderer;
	utils::Image destTexture;
	textureRenderer.SetSourceNoiseMap(noiseMap);
	textureRenderer.SetDestImage(destTexture);
	textureRenderer.EnableLight(false);
	textureRenderer.Render();
  
	utils::Color *c=destTexture.GetSlabPtr();

	int i;
	for(i = 0; i < _height; ++i)
		bcopy(c + i * _width, (unsigned char *)_scratchBufferPtr + i * _scratchBufferRowBytes, _width*4);

	CGContextRef  bitmapContext = CGBitmapContextCreate(_scratchBufferPtr, _width, _height, 8 ,_scratchBufferRowBytes, _colorspace, kCGImageAlphaPremultipliedFirst);
	CGImageRef    bitmapImage = CGBitmapContextCreateImage(bitmapContext);
	QCGLCGImage *qcglOut=[[QCGLCGImage alloc] initWithCGImage:bitmapImage options:0];
	CGImageRelease(bitmapImage);
	CGContextRelease(bitmapContext);

	return qcglOut;
}

-(void) dealloc
{
	CGColorSpaceRelease(_colorspace);
	if(_scratchBufferPtr)
		free(_scratchBufferPtr);
	if(_scratchBufferPtrFlip)
		free(_scratchBufferPtrFlip);
	
	[super dealloc];
}
@end